<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2012-2014 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * @author Karsten Deubert <info@deubert.it>
 */
class DZF_Mail extends \Zend_Mail
{
    /**
     * @var string
     */
    protected $_bodyHtmlRaw = '';

    /**
     * @var string
     */
    protected $_bodyTextRaw = '';

    /**
     * @var string
     */
    protected $_subjectRaw = '';

    public function setSubject($subject)
    {
        $this->_subjectRaw = $subject;

        return parent::setSubject($subject);
    }

    public function getSubjectRaw()
    {
        return $this->_subjectRaw;
    }

    public function setBodyHtml($html, $charset = null, $encoding = \Zend_Mime::ENCODING_QUOTEDPRINTABLE)
    {
        $this->_bodyHtmlRaw = $html;

        return parent::setBodyHtml($html, $charset, $encoding);
    }

    public function setBodyText($txt, $charset = null, $encoding = \Zend_Mime::ENCODING_QUOTEDPRINTABLE)
    {
        $this->_bodyTextRaw = $txt;

        return parent::setBodyText($txt, $charset, $encoding);
    }

    public function getBodyHtmlRaw()
    {
        return $this->_bodyHtmlRaw;
    }

    public function getBodyTextRaw()
    {
        return $this->_bodyTextRaw;
    }

    public static function mailException(\Exception $exception)
    {
        $mailContent  = 'generated via DZF_Mail::mailException for '.APPLICATION_NAME.' in ENV '.APPLICATION_ENV.' on '.date('d.m.Y H:i:s', time())."\n";

        $mailContent .= "\n";
        $mailContent .= "{code}\n";

        $mailContent .= 'exception type: '.get_class($exception)."\n\n";
        $mailContent .= 'exception message: '.$exception->getMessage()."\n\n";
        $mailContent .= 'exception trace: '.$exception->getTraceAsString()."\n\n";
        $mailContent .= '$_REQUEST: '.var_export($_REQUEST, true)."\n\n";
        $mailContent .= '$_GET: '.var_export($_GET, true)."\n\n";
        $mailContent .= '$_POST: '.var_export($_POST, true)."\n\n";
        $mailContent .= '$_SERVER: '.var_export($_SERVER, true)."\n\n";
        $mailContent .= '$_COOKIE: '.var_export($_COOKIE, true)."\n\n";
        $mailContent .= '$_SESSION: '.var_export(DZF_Exception::filterSession($_SESSION), true)."\n\n";

        if ($exception instanceof DZF_Exception && isset($exception->debugData))
        {
            $mailContent .= 'debugData: '.$exception->debugData."\n\n";
        }

        $mailContent .= "\n";
        $mailContent .= "{code}\n";

        $exceptionMail = new \Zend_Mail('utf-8');
        $exceptionMail->setSubject('['.APPLICATION_ENV.'] '.APPLICATION_NAME.' exception mail');
        $exceptionMail->setBodyText($mailContent);
        $exceptionMail->addTo(\Zend_Registry::get('debugging.emailtarget'));
        $exceptionMail->send();
    }

    /**
     * @param string $content
     * @param string $pgpPublicKeyRaw
     * @param string $ourPrivateKeyRaw if set will be used to sign the content before encryption
     * @param string $ourPrivateKeyPassword if set will be used to sign the content before encryption
     * @return string
     * @throws DZF_Exception
     */
    public static function getPGPEnarmoredContent($content, $pgpPublicKeyRaw, $ourPrivateKeyRaw = null, $ourPrivateKeyPassword = null)
    {
        set_time_limit(120);

        $data = new \OpenPGP_LiteralDataPacket($content);

        if (!is_null($ourPrivateKeyRaw))
        {
            $keyEncrypted = \OpenPGP_Message::parse(\OpenPGP::unarmor($ourPrivateKeyRaw, 'PGP PRIVATE KEY BLOCK'));

            $foundKey = false;
            $ourPrivateKey = null;

            foreach ($keyEncrypted as $p)
            {
                if(!($p instanceof \OpenPGP_SecretKeyPacket)) continue;

                $ourPrivateKey = \OpenPGP_Crypt_Symmetric::decryptSecretKey($ourPrivateKeyPassword, $p);

                if (!is_null($ourPrivateKey))
                {
                    $foundKey = true;
                    break;
                }
            }

            if (!$foundKey)
            {
                throw new DZF_Exception('pgp private key packet not found');
            }

            if (is_null($ourPrivateKey))
            {
                throw new DZF_Exception('pgp private key parse error');
            }

            if (!$ourPrivateKey instanceof \OpenPGP_SecretKeyPacket)
            {
                throw new DZF_Exception(sprintf('wrong private key class: %s', get_class($ourPrivateKey)));
            }
            if (!$ourPrivateKey instanceof \OpenPGP_PublicKeyPacket)
            {
                throw new DZF_Exception(sprintf('wrong private key class: %s', get_class($ourPrivateKey)));
            }

            if ($ourPrivateKey->tag != 5)
            {
                /* @see OpenPGP_Packet::$tags - 5 => 'SecretKey'*/
                throw new DZF_Exception(sprintf('invalid tag for private key packet %d', $ourPrivateKey->tag));
            }

            if (!isset($ourPrivateKey->key['d']))
            {
                throw new DZF_Exception('d value missing in private key container');
            }

            $sign = new \OpenPGP_Crypt_RSA($ourPrivateKey);
            $signedData = $sign->sign($data);
            if (is_null($signedData))
            {
                throw new DZF_Exception('pgp message signing error');
            }
        }
        else
        {
            $signedData = $data;
        }

        if ($recipientKeyUnarmored = \OpenPGP::unarmor($pgpPublicKeyRaw, 'PGP PUBLIC KEY BLOCK'))
        {
            $recipientKey = \OpenPGP_PublicKeyPacket::parse($recipientKeyUnarmored);
        }
        else
        {
            $recipientKey = \OpenPGP_PublicKeyPacket::parse($pgpPublicKeyRaw);
        }

        if (is_null($recipientKey))
        {
            throw new DZF_Exception('pgp public key parse error');
        }

        if (!$recipientKey instanceof \OpenPGP_PublicKeyPacket)
        {
            throw new DZF_Exception(sprintf('wrong public key class: %s', get_class($recipientKey)));
        }

        $signedMessage = new \OpenPGP_Message(array($signedData));
        $encryptedContent = \OpenPGP_Crypt_Symmetric::encrypt($recipientKey, $signedMessage);
        $enarmoredContent = \OpenPGP::enarmor($encryptedContent->to_bytes(), 'PGP MESSAGE', array('Version' => 'DZF'));

        return $enarmoredContent;
    }

    /**
     * @param $content
     * @param $pgpPublicKeyRaw
     * @return string
     * @throws DZF_Exception
     */
    public static function encrypt($content, $pgpPublicKeyRaw)
    {
        set_time_limit(120);

        if ($recipientKeyUnarmored = \OpenPGP::unarmor($pgpPublicKeyRaw, 'PGP PUBLIC KEY BLOCK'))
        {
            $recipientKey = OpenPGP_Message::parse($recipientKeyUnarmored);
        }
        else
        {
            $recipientKey = OpenPGP_Message::parse($pgpPublicKeyRaw);
        }

        if ($recipientKey === null)
        {
            throw new DZF_Exception('pgp public key parse error');
        }

        if (!$recipientKey instanceof \OpenPGP_Message)
        {
            throw new DZF_Exception(sprintf('wrong public key class: %s', get_class($recipientKey)));
        }

        $data = new OpenPGP_LiteralDataPacket($content, array('format' => 'u'));
        $encrypted = OpenPGP_Crypt_Symmetric::encrypt($recipientKey, new OpenPGP_Message(array($data)));
        $enarmoredContent = \OpenPGP::enarmor($encrypted->to_bytes(), 'PGP MESSAGE', array('Version' => 'DZF'));

        return $enarmoredContent;
    }

    /**
     * @param $encryptMessage
     * @param $armoredPrivateKey
     * @param $privateKeyPassPhrase
     * @return array
     * @throws Exception
     */
    public static function decrypt($encryptMessage, $armoredPrivateKey, $privateKeyPassPhrase)
    {
        set_time_limit(120);

        $resultDecrypted = array();

        $keyEncrypted = OpenPGP_Message::parse(OpenPGP::unarmor($armoredPrivateKey, 'PGP PRIVATE KEY BLOCK'));

        // Try each secret key packet
        foreach($keyEncrypted as $p) {
            if(!($p instanceof OpenPGP_SecretKeyPacket)) continue;
            $key = OpenPGP_Crypt_Symmetric::decryptSecretKey($privateKeyPassPhrase, $p);
            $msg = OpenPGP_Message::parse(OpenPGP::unarmor($encryptMessage, 'PGP MESSAGE'));
            $decryptor = new OpenPGP_Crypt_RSA($key);
            $decrypted = $decryptor->decrypt($msg);
            if ($decrypted !== null) {
                $resultDecrypted[] = $decrypted;
            }
        }

        return $resultDecrypted;
    }

    /**
     * @param $encryptMessage
     * @param $armoredPrivateKey
     * @param $privateKeyPassPhrase
     * @return string
     * @throws Exception
     */
    public static function sign($encryptMessage, $armoredPrivateKey, $privateKeyPassPhrase)
    {
        $keyEncrypted = OpenPGP_Message::parse(OpenPGP::unarmor($armoredPrivateKey, 'PGP PRIVATE KEY BLOCK'));
        $keyEncrypted = $keyEncrypted[0];

        $key = OpenPGP_Crypt_Symmetric::decryptSecretKey($privateKeyPassPhrase, $keyEncrypted);
        $data = OpenPGP_Message::parse($encryptMessage);
        $decryptor = new OpenPGP_Crypt_RSA($key);
        $signMessage = $decryptor->sign($data);

        $enarmoredSignContent = \OpenPGP::enarmor($signMessage->to_bytes(), 'PGP MESSAGE', array('Version' => 'DZF'));

        return $enarmoredSignContent;
    }
}
